#!/usr/bin/env python
"""provides functions to look up potentials on an apbs grid

   requires a file name as an argument if used on its own

   Copyright 2005, Mark Engelhardt (marke@stanford.edu)
"""
import isim_grid_pot as igp
from vgrid import * # this module is provided by apbs
import sys
import math
from sys import stdout, stderr

class APBS_grid:
  
  def __init__(self, infile):
    """converts a grid object from the vgrid module to a list


    this is all redundant - don't need to redefine the grid
    perhaps recode sometime
    """
    startVio()
    data = double_array(3)
    stdout.write("main:  Reading data from %s... \n" % infile)
    grid = Vgrid_ctor(0, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, data)
    Vgrid_readDX(grid, "FILE", "ASC", "", infile)
    
    self.grid = grid
    self.nx = grid.nx
    self.ny = grid.ny
    self.nz = grid.nz
    self.xmin = grid.xmin
    self.ymin = grid.ymin
    self.zmin = grid.zmin
    self.hx = grid.hx
    self.hy = grid.hy
    self.hz = grid.hzed


  def PotLookup(self, coords, cutoff = 3):
    """returns the potential at x,y,z from the average of the surrounding points

       the cutoff determines the radius from the point used in averaging
    """

    total_weights = 0
    sum_inverses = 0
    num = 0
    w = -1
    pt = double_array(3)
    val = ptrcreate("double", 0.0)
    
    # these will spit out the coordinate points in the same order that they were
    # put into the grid list
    for i in range(self.nx):
      for j in range(self.ny):
        for k in range(self.nz):
          x = self.xmin + i*self.hx
          y = self.ymin + j*self.hy
          z = self.zmin + k*self.hz
          
          if abs(coords[0] - x) > cutoff: continue
          if abs(coords[1] - y) > cutoff: continue
          if abs(coords[2] - z) > cutoff: continue
          
          d = igp.Distance(coords, (x,y,z))
          if d <= cutoff:
            set_entry(pt, 0, x)
            set_entry(pt, 1, y)
            set_entry(pt, 2, z)
            if Vgrid_value(self.grid, pt, val):
              pot = ptrvalue(val)
            else:
              stderr.write("Bad grid value at position %d %d %d.\n" % (i, j, k))
              stderr.write("No valid point for PotLookup.  Skipping.")
              continue
            num += 1
            w = 1/d**2
            total_weights += w
            sum_inverses += w * pot
#    print "summing over %d points for %s" % (num, coords)
    if w == -1:
      stderr.write("PotLookup returned no valid points.\n")
      ptrfree(val)
      return 0.0
    ptrfree(val)
    return sum_inverses / total_weights

def main():
  coords = [0.0, 0.00873, 35.65]

  infile = sys.argv[1]

  apbs_grid = APBS_grid(infile)
  pot = apbs_grid.PotLookup(coords)
  print pot

        
if __name__ == "__main__":
  main()
